#!/usr/bin/python3
# -*- coding: UTF-8 -*-
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk as gtk
from gi.repository import GLib as glib
from gi.repository import GdkPixbuf
import os
import os.path
import sys
import time
import INA219

IMAGE_LOC = os.path.join(os.path.dirname(sys.argv[0]), "images/battery")

class BatteryTray:
    def __init__(self):
        self.ina=INA219.INA219(addr=0x43)
        self.charge = 0
        self.counter = 0
        self.warning = None
        self.timeout_id = None
        
        self.tray = gtk.StatusIcon()
        # self.tray.connect('activate', self.refresh)

        # Create menu
        self.menu = gtk.Menu()
        
        i = gtk.MenuItem(label="About...")
        i.show()
        i.connect("activate", self.show_about)
        self.menu.append(i)
        i = gtk.MenuItem(label="Quit")
        i.show()
        i.connect("activate", self.quit)
        self.menu.append(i)
        self.tray.connect('popup-menu', self.show_menu)

        # Initalise and start battery display
        self.refresh(None)
        self.tray.set_visible(True)
        glib.timeout_add(1000, self.refresh, self.tray)  #1s

    def show_menu(self, widget, event_button, event_time):
        self.menu.popup(None, None,
            self.tray.position_menu,
            self.tray,
            event_button,
            gtk.get_current_event_time()
        )

    def show_about(self, widget):
        dialog = gtk.AboutDialog()
        logo = GdkPixbuf.Pixbuf.new_from_file('images/logo.png')
        dialog.set_logo(logo)
        dialog.set_program_name('Battery Monitor Demo')
        dialog.set_version('Version: v1.0')
        dialog.set_website_label('WaveShare Official Website')
        dialog.set_website('https://www.waveshare.com/')
        dialog.set_authors(['WaveShare Team'])
        gtk_version = '{}.{}.{}'.format(gtk.get_major_version(),gtk.get_minor_version(), gtk.get_micro_version())
        comment = '{}\n\n'.format("It's a battery Display By waveshare")
        comment += 'Gtk: {}'.format(gtk_version)
        dialog.set_comments(comment)
        dialog.run()
        dialog.destroy() 

    def quit(self, widget):
        gtk.main_quit()

    def on_timeout(self, *args, **kwargs):
        """ Handles destroy event. """
        # print("countdown")
        self.counter -= 0.2
        if(self.counter > 0):  #countdown
            current = -int(self.ina.getCurrent_mA())                   # current in mA
            if(current > 50):
                self.charge = 1
                self.warning.response(0)
            else:
                self.charge = 0
                self.warning.format_secondary_text("auto shutdown after " +str(int(self.counter)) + " seconds")
                self.warning.show_all()
        else:   #timeout
            address = os.popen("i2cdetect -y -r 1 0x2d 0x2d | egrep '2d' | awk '{print $2}'").read()
            if(address=='2d\n'):
                #print("If charged, the system can be powered on again.")
                #write 0x55 to 0x01 register of 0x2d Address device
                os.popen("i2cset -y 1 0x2d 0x01 0x55")
            os.system("sudo poweroff")
        return True

    def refresh(self, widget):
        # print("refresh")
        bus_voltage = self.ina.getBusVoltage_V()             # voltage on V- (load side)
        current = -int(self.ina.getCurrent_mA())                   # current in mA
        if(current > 50):self.charge = 1
        else:self.charge = 0

        b_level = int((bus_voltage - 3)/1.2*100)
        if(b_level > 100):b_level = 100
        if(b_level < 0):b_level = 0
        # print("bus_voltage = ",bus_voltage,"  current = ",current,str(int(b_level / 10 + self.charge * 11)))

        b_file = IMAGE_LOC + "." + str(int(b_level / 10 + self.charge * 11)) + ".png"
        self.tray.set_tooltip_text("%d%%  %.1fV  %dmA" % (b_level,bus_voltage,current))

        if os.path.exists(b_file):
            self.tray.set_from_file(b_file)
        else:
            print(b_file)
            print('icon file not exist')

        if(b_level <= 10 and self.charge == 0):
            # print("WARNING")
            self.counter = 60
            self.warning = gtk.MessageDialog(
                parent=None,
                modal=True,
                message_type=gtk.MessageType.WARNING,
                # buttons=gtk.ButtonsType.none,
                text="""
                The battery level is below
                Please connect in the power adapter
                """)
            self.warning.set_title("Battery Warning")
            self.warning.format_secondary_text("auto shutdown after " +str(int(self.counter)) + " seconds")
            scoreimg = gtk.Image ()
            scoreimg.set_from_file("images/batteryQ.png")
            self.warning.set_image(scoreimg)

            self.timeout_id = glib.timeout_add(200, self.on_timeout, self.warning)
            self.warning.connect('delete-event',lambda widget, event: True)
            self.warning.show_all()
            self.warning.run()
            glib.source_remove(self.timeout_id)
            self.warning.destroy()
        return True

if __name__ == '__main__':
    app = BatteryTray()
    gtk.main()
